/*
 * Copyright (C) 2009-2010 Eiichiro Uchiumi. All Rights Reserved.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 *     
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.eiichiro.reverb.diagnostics;

import java.io.PrintStream;
import java.lang.reflect.Field;
import java.util.Arrays;

/**
 * {@code VariableDump} dumps the state of target variable object.
 *
 * @author <a href="mailto:eiichiro@eiichiro.org">Eiichiro Uchiumi</a>
 */
public class VariableDump implements Dump {

	private static final String TREE = "+ ";
	
	private static final String INDENT = "  ";
	
	private final Object variable;
	
	/**
	 * Constructs a new {@code VariableDump} instance with the specified variable 
	 * object.
	 * 
	 * @param variable The variable object to be dumped.
	 */
	public VariableDump(Object variable) {
		if (variable == null) {
			throw new IllegalArgumentException("'variable' must not be [" + variable + "]");
		}
		
		this.variable = variable;
	}

	/**
	 * Dumps the state of {@code VariableDump#variable} to the specified {@code PrintStream}.
	 * 
	 * @see Dump#dump(PrintStream)
	 */
	public void dump(PrintStream printStream) {
		printStream.print(toString());
	}
	
	@Override
	public String toString() {
		return toString(variable, variable.getClass(), "", 0).toString();
	}
	
	private StringBuilder toString(Object variable, Class<?> type, String name, int depth) {
		StringBuilder stringBuilder = new StringBuilder();
		String string;
		
		if (variable == null) {
			string = null;
		} else if (variable.getClass().isArray()) {
			string = Arrays.toString((Object[]) variable);
		} else {
			string = variable.toString();
		}
		
		stringBuilder.append(TREE + "[" + name + ": " + type.getName() + "] " + string);
		
		if (variable != null && string.equals(type.getName() + "@" + Integer.toHexString(variable.hashCode()))) {
			int d = ++depth;
			
			for (Field field : variable.getClass().getDeclaredFields()) {
				Object value = null;
				
				try {
					// :-)
					field.setAccessible(true);
					value = field.get(variable);
				} catch (Exception e) {
					value = e;
				}
				
				stringBuilder.append("\n");
				
				for (int i = 0; i < depth; i++) {
					stringBuilder.append(INDENT);
				}
				
				stringBuilder.append(toString(value, field.getType(), field.getName(), d));
			}
		}
		
		return stringBuilder;
	}
	
}
